from setuptools import setup

setup(name='muqfatc',
      version='0.1',
      description='Image analysis tool for analysing QFA (Addinall et al., 2011) cultures on a high-throughput level. Single lineage and pin population growth is catpured.',
      url='https://github.com/lwlss/discstoch',
      author='Helena Herrmann',
      author_email='h.herrmann2@gmail.com',
      license='MIT',
      packages=['muqfatc'],
      zip_safe=False)
